@@ -1,17 +1,23 @@
 package Kelp::Module::JSON;
 
-use Kelp::Base 'Kelp::Module';
+use Kelp::Base 'Kelp::Module::Encoder';
 
 use JSON::MaybeXS;
 
+sub encoder_name { 'json' }
+
+sub build_encoder
+{
+    my ($self, $args) = @_;
+    return JSON::MaybeXS->new(%$args);
+}
+
 sub build
 {
     my ($self, %args) = @_;
-    my $json = JSON::MaybeXS->new(%args);
-    my $json_internal = JSON::MaybeXS->new(%args, utf8 => 0);
+    $self->SUPER::build(%args);
 
-    $self->register(json => $json);
-    $self->register(_json_internal => $json_internal);
+    $self->register(json => $self->get_encoder);
 }
 
 1;
@@ -31,15 +37,20 @@ Kelp::Module::JSON - Simple JSON module for a Kelp application
         my $self = shift;
 
         # manually render a json configured to UTF-8
-        $self->res->json->set_charset('UTF-8');
+        $self->res->set_charset('UTF-8');
         $self->res->render_binary(
             $self->json->encode({ yes => 1 })
         );
     }
 
+=head1 DESCRIPTION
+
+Standard JSON encoder/decoder. Chooses the best backend through L<JSON::MaybeXS>.
+
 =head1 REGISTERED METHODS
 
-This module registers only one method into the application: C<json>.
+This module registers only one method into the application: C<json>. It also
+registers itself for later use by L<Kelp/get_encoder> under the name C<json>.
 
 The module will try to use backends in this order: I<Cpanel::JSON::XS, JSON::XS, JSON::PP>.
 
@@ -49,7 +60,7 @@ You should probably not use C<utf8>, and just encode the value into a proper
 charset by hand. You may not always want to have encoded strings anyway, for
 example some interfaces may encode the values themselves.
 
-Kelp will register a second JSON encoder / decoder with all the same options
+Kelp will use an internal copy of JSON encoder / decoder with all the same options
 but without C<utf8>, reserved for internal use. Modifying C<json> options at
 runtime will not cause the request / response encoding to change.
 
