@@ -4,8 +4,8 @@ use Kelp::Base 'Plack::Request';
 
 use Carp;
 use Try::Tiny;
-use Encode qw(encode decode);
 use Hash::MultiValue;
+use Kelp::Util;
 
 attr -app => sub { croak "app is required" };
 
@@ -15,6 +15,21 @@ attr stash => sub { {} };
 # The named hash contains the values of the named placeholders
 attr named => sub { {} };
 
+# charset which came with the request
+attr -charset => sub {
+    my $self = shift;
+
+    return undef unless $self->content_type;
+    return undef unless $self->content_type =~ m{
+        ^(?:                  # only on some content-types
+            text/ | application/
+        )
+        .+
+        ;\s*charset=([^;\$]+) # get the charset
+    }xi;
+    return $1;
+};
+
 # The name of the matched route for this request
 attr route_name => sub { undef };
 
@@ -22,25 +37,25 @@ attr query_parameters => sub {
     my $self = shift;
 
     $self->SUPER::query_parameters;
-    my $raw = $self->_charset_decode_array($self->env->{'plack.request.query_parameters'});
-    return Hash::MultiValue->new(@{$raw});
+    my $decoded = $self->_charset_decode_array($self->env->{'plack.request.query_parameters'}, 1);
+    return Hash::MultiValue->new(@{$decoded});
 };
 
 attr body_parameters => sub {
     my $self = shift;
 
     $self->SUPER::body_parameters;
-    my $raw = $self->_charset_decode_array($self->env->{'plack.request.body_parameters'});
-    return Hash::MultiValue->new(@{$raw});
+    my $decoded = $self->_charset_decode_array($self->env->{'plack.request.body_parameters'});
+    return Hash::MultiValue->new(@{$decoded});
 };
 
 attr parameters => sub {
     my $self = shift;
 
     $self->SUPER::parameters;
-    my $raw_query = $self->_charset_decode_array($self->env->{'plack.request.query_parameters'});
-    my $raw_body = $self->_charset_decode_array($self->env->{'plack.request.body_parameters'});
-    return Hash::MultiValue->new(@{$raw_query}, @{$raw_body});
+    my $decoded_query = $self->_charset_decode_array($self->env->{'plack.request.query_parameters'}, 1);
+    my $decoded_body = $self->_charset_decode_array($self->env->{'plack.request.body_parameters'});
+    return Hash::MultiValue->new(@{$decoded_query}, @{$decoded_body});
 };
 
 # Raw methods - methods in Plack::Request (without decoding)
@@ -105,57 +120,30 @@ sub is_json
     return $self->content_type =~ m{^application/json}i;
 }
 
-sub charset
-{
-    my $self = shift;
-
-    return undef unless $self->content_type;
-    return undef unless $self->content_type =~ m{;\s*charset=([^;\$]+)}i;
-    return $1;
-}
-
-sub effective_charset
-{
-    my $self = shift;
-
-    # charset must be supported by Encode
-    state $supported = {map { lc $_ => $_ } Encode->encodings(':all')};
-
-    # try to get the charset from the request
-    my $charset = $self->charset;
-    $charset = $supported->{lc $charset} if $charset;
-
-    # if no charset or unsupported, app's charset will be used
-    return $charset || $self->app->charset;
-}
-
-sub charset_encode
-{
-    my ($self, $string) = @_;
-
-    return encode $self->effective_charset, $string;
-    return $string unless $self->effective_charset;
-}
-
 sub charset_decode
 {
-    my ($self, $string) = @_;
+    my ($self, $string, $configured_only) = @_;
+    my $req_charset = $self->app->request_charset;
 
-    return $string unless $self->effective_charset;
-    return decode $self->effective_charset, $string;
+    # do not decode at all if the application is set no not decode
+    return $string unless $req_charset;
+    return Kelp::Util::charset_decode(
+        (!$configured_only && Kelp::Util::effective_charset($self->charset)) || $req_charset,
+        $string,
+    );
 }
 
 sub _charset_decode_array
 {
-    my ($self, $arr) = @_;
+    my ($self, $arr, $configured_only) = @_;
 
-    return [map { $self->charset_decode($_) } @$arr];
+    return [map { $self->charset_decode($_, $configured_only) } @$arr];
 }
 
 sub path
 {
     my $self = shift;
-    return $self->charset_decode($self->SUPER::path(@_));
+    return Kelp::Util::charset_decode($self->app->request_charset, $self->SUPER::path(@_));
 }
 
 sub content
@@ -269,38 +257,55 @@ it to add several convenience methods and support for application encoding.
 
 =head1 ENCODING
 
-Starting with version 2.01, Kelp::Request simplifies input handling and
-improves correctness by automatically decoding path, query parameters and body
-parameters.
+Starting with version 2.10, Kelp::Request simplifies input handling and
+improves correctness by automatically decoding path, query parameters and content.
 
 Headers (so cookies as well) are unaffected, as they aren't consistently
 supported outside of ASCII range. JSON now decodes request data into the proper
-charset instead of flat utf8 if so configured. Sessions are configured
-separately in middlewares, so they must themselves do the proper decoding.
+charset instead of flat utf8 regardless of configuration. Sessions are
+configured separately in middlewares, so they must themselves do the proper
+decoding.
 
 Following methods will return values decoded with charset either from
-C<Content-Type> header or the one specified in the app's configuration:
+C<Content-Type> header or the one specified in the app's configuration
+(L<Kelp/request_charset>):
 
 =over
 
-=item * C<path>
-
 =item * C<param>
 
 =item * C<cgi_param>
 
-=item * C<query_param>
-
 =item * C<body_param>
 
-=item * C<parameters>
+=item * C<json_param>
 
-=item * C<query_parameters>
+=item * C<parameters>
 
 =item * C<body_parameters>
 
 =item * C<content>
 
+=item * C<json_content>
+
+=back
+
+Following methods will always decode to L<Kelp/request_charset> because they
+are not the part of message's content (URIs should always be in ASCII-compilant
+encoding, UTF-8 is preferable):
+
+=over
+
+=item * C<path>
+
+=item * C<param> (from query)
+
+=item * C<cgi_param> (from query)
+
+=item * C<parameters> (from query)
+
+=item * C<query_parameters>
+
 =back
 
 If you wish to get input in the original request encoding, use these instead
@@ -325,21 +330,32 @@ are configured to decode them:
 
 =over
 
-=item * C<param> - depends on JSON module (on JSON requests)
-
-=item * C<json_param> - depends on JSON module
-
-=item * C<json_content> - depends on JSON module
-
 =item * C<session> - depends on session middleware
 
 =back
 
+B<Some caveats> about the automatic decoding and L<Kelp/request_charset>
+configuration parameter:
+
+As always, UTF-8 (the default) works best - don't change to avoid issues. Other
+ASCII-compilant encodings should work well. L</content> will always be decoded
+properly, but C<application/x-www-form-urlencoded> and C<multipart/form-data>
+will have issues with non-ASCII-compilant encodings. Especially the latter,
+because the information about C<Content-Type> of a single part is lost on Plack
+level and it is not properly decoded using that encoding. In such corner cases,
+you should probably get the full undecoded body using L</raw_body> and parse it
+yourself.
+
+If you wish to disable automatic decoding, you can set L<Kelp/request_charset>
+to undef - it will then ignore any charset which came with the message and let
+you do your own decoding.
+
 =head1 ATTRIBUTES
 
 =head2 app
 
-A reference to the Kelp application.
+A reference to the Kelp application. This will always be the real application,
+not the reblessed controller.
 
 =head2 stash
 
@@ -369,6 +385,12 @@ current route is processing.
 Contains a string name of the route matched for this request. Contains route pattern
 if the route was not named.
 
+=head2 charset
+
+Returns the charset from the C<Content-Type> HTTP header or C<undef> if there
+is none. Will ignore the charset unless C<Content-Type> is C<text/*> or
+C<application/*>. Readonly.
+
 =head1 METHODS
 
 =head2 param
@@ -549,26 +571,13 @@ Returns true if the request was called with C<XMLHttpRequest>.
 
 Returns true if the request's content type was C<application/json>.
 
-=head2 charset
-
-Returns the charset from the C<Content-Type> HTTP header or C<undef> if there
-is none.
-
-=head2 effective_charset
-
-Like L</charset>, but also checks whether the charset is supported by Encode.
-If there is no charset in request or it isn't supported, L<Kelp/charset> will
-be returned.
-
 =head2 charset_decode
 
-Same as L<Kelp/charset_decode>, but will prefer using L</charset> to L<Kelp/charset>.
+Shortcut method, which decodes a string using L</charset> or
+L<Kelp/request_charset>. A second optional parameter can be passed, and if true
+will cause the method to ignore charset passed in the C<Content-Type> header.
 
-=head2 charset_encode
-
-Encoding counterpart of L</charset_decode>. It's only useful in very narrow
-scenario, so avoid using it - usually you want to encode into response's
-charset, which will be an application charset.
+It does noting if L<Kelp/request_charset> is undef or false.
 
 =cut
 
