@@ -8,6 +8,7 @@ use Kelp::Util;
 use Kelp::Routes::Location;
 use Try::Tiny;
 use Class::Inspector;
+use Scalar::Util qw(weaken);
 
 attr base => '';    # the default is set by config module
 attr rebless => 0;    # do not rebless app by default
@@ -34,6 +35,9 @@ sub add
     $parent = {} if !$parent || ref $parent ne 'HASH';
 
     my $route = $self->_parse_route($parent, $pattern, $descr);
+
+    Kelp::Base::_DEBUG(routes => 'Added route: ', $route);
+
     return $self->_build_location($route);
 }
 
@@ -269,10 +273,9 @@ sub wrap_psgi
 # Override to use a custom pattern object
 sub build_pattern
 {
-    my ($self, $args) = @_;
-    my $package = $self->pattern_obj;
-    eval qq{require $package};
-    return $package->new(%$args);
+    return Kelp::Util::load_package($_[0]->pattern_obj)->new(
+        %{$_[1]}
+    );
 }
 
 sub match
@@ -324,7 +327,7 @@ sub dispatch
         unless $dest;
 
     my ($to, $controller, $action) = ($route->to, @{$dest});
-    $app = $app->_clone($controller) if $controller;
+    $app = $app->context->set_controller($controller) if $controller;
 
     $app->before_dispatch($to);
     return $action->($app, @{$route->param});
@@ -948,7 +951,8 @@ Override this method to change the way a Plack/PSGI application is extracted fro
 
 This is the default router class for each new Kelp application, but it doesn't
 have to be. You can create your own subclass that better suits your needs. It's
-generally enough to override the L</dispatch> or L</format_to> method.
+generally enough to override the L</dispatch>, L</format_to> or
+L</load_destination> methods.
 
 =head1 ACKNOWLEDGEMENTS
 
