@@ -5,11 +5,13 @@ use Carp;
 use Encode qw();
 
 # improve error locations of croak
-our @CARP_NOT = (qw(
-    Kelp
-    Kelp::Routes
-    Kelp::Context
-));
+our @CARP_NOT = (
+    qw(
+        Kelp
+        Kelp::Routes
+        Kelp::Context
+    )
+);
 
 sub camelize
 {
