@@ -1,7 +1,8 @@
 package MyApp::Routes2;
 
-sub goodbye {
-    my ( $self, $name ) = @_;
+sub goodbye
+{
+    my ($self, $name) = @_;
     return "BYE $name";
 }
 
